uniform extern float4x4 gWVP : WorldViewProjection;
uniform extern texture gTexture0;
uniform extern float4x4 gTexTransform;
uniform extern float gOpacity = 1.0;
static const float PixelSizeX = 0.0015;
static const float PixelSizeY = 0.0015;

#include <PostProcessing.Base.fx>

sampler TextureSampler0 = sampler_state
{
  Texture = <gTexture0>;
  MinFilter = LINEAR;
  MagFilter = LINEAR;
  MipFilter = LINEAR;
  AddressU = CLAMP;
  AddressV = CLAMP;
};

struct OutputVS
{
	float4 posH: POSITION0;
	float2 tex0: TEXCOORD0;
};

OutputVS mainVS(float3 pos : POSITION, float2 tex0: TEXCOORD0) {
	OutputVS outVS;
	
	outVS.posH = mul(float4(pos.xyz, 1.0f), gWVP);
	outVS.tex0 = mul(float3(cliptexturecoordinates(tex0), 1.0f), gTexTransform).xy;
	
	return outVS;
}



float4 Blur9PS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor = tex2D(TextureSampler0, tex0);
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(-PixelSizeX, -PixelSizeY)) * 0.5;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(0, -PixelSizeY)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX, -PixelSizeY)) * 0.5;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(-PixelSizeX, 0)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX, 0)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(-PixelSizeX, +PixelSizeY)) * 0.5;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(0, +PixelSizeY)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX, +PixelSizeY)) * 0.5;
  texColor = texColor / (1 + 4 * 0.5 + 4 * 0.8);
  
  return float4(texColor.rgb, texColor.a * gOpacity);
}

float4 Blur25PS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor = (float4)0;
	
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -2, PixelSizeY * -2)) * 0.6;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -1, PixelSizeY * -2)) * 0.7;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +0, PixelSizeY * -2)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +2, PixelSizeY * -2)) * 0.7;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +1, PixelSizeY * -2)) * 0.6;

  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -2, PixelSizeY * -1)) * 0.7;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -1, PixelSizeY * -1)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +0, PixelSizeY * -1)) * 0.9;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +2, PixelSizeY * -1)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +1, PixelSizeY * -1)) * 0.7;
  
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -2, PixelSizeY * +0)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -1, PixelSizeY * +0)) * 0.9;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +0, PixelSizeY * +0)) * 1.0;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +2, PixelSizeY * +0)) * 0.9;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +1, PixelSizeY * +0)) * 0.8;
  
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -2, PixelSizeY * +1)) * 0.7;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -1, PixelSizeY * +1)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +0, PixelSizeY * +1)) * 0.9;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +2, PixelSizeY * +1)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +1, PixelSizeY * +1)) * 0.7;

  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -2, PixelSizeY * +2)) * 0.6;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * -1, PixelSizeY * +2)) * 0.7;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +0, PixelSizeY * +2)) * 0.8;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +2, PixelSizeY * +2)) * 0.7;
  texColor = texColor + tex2D(TextureSampler0, tex0 + float2(PixelSizeX * +1, PixelSizeY * +2)) * 0.6;

  texColor = texColor / (1 + 4 * 0.6 + 8 * 0.7 + 8 * 0.8 + 4 * 0.9);
  
  return float4(texColor.rgb, texColor.a * gOpacity);
}


technique Blur9 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 Blur9PS();
		// FillMode = wireframe;
		FillMode = solid;

		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique Blur25 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 Blur25PS();
		// FillMode = wireframe;
		FillMode = solid;

		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}