uniform extern float4x4 gWVP : WorldViewProjection;
uniform extern texture gTexture0;
uniform extern texture gTexture1;
uniform extern float4x4 gTexTransform;
uniform extern float gOpacity = 1.0;
uniform extern float4 gChannels = float4(1, 1, 0.4, 1);

#include <PostProcessing.Base.fx>

sampler TextureSampler0 = sampler_state
{
  Texture = <gTexture0>;
  MinFilter = LINEAR;
  MagFilter = LINEAR;
  MipFilter = LINEAR;
  AddressU = MIRROR;
  AddressV = MIRROR;
};

sampler TextureSampler1 = sampler_state
{
  Texture = <gTexture1>;
  MinFilter = LINEAR;
  MagFilter = LINEAR;
  MipFilter = LINEAR;
  AddressU = MIRROR;
  AddressV = MIRROR;
};

struct OutputVS
{
	float4 posH: POSITION0;
	float2 tex0: TEXCOORD0;
};

OutputVS mainVS(float3 pos : POSITION, float2 tex0: TEXCOORD0) {
	OutputVS outVS;
	
	outVS.posH = mul(float4(pos.xyz, 1.0f), gWVP);
	outVS.tex0 = mul(float3(cliptexturecoordinates(tex0), 1.0f), gTexTransform).xy;
	
	return outVS;
}

float4 MaskPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = texColor2 * gChannels;
  float alpha = max(max(texColor.r, texColor.g), max(texColor.b, texColor.a));
  texColor = tex2D(TextureSampler0, tex0);
  
  return float4(texColor.rgb, texColor.a * alpha * gOpacity);
}

float4 MaskInvertedPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = (texColor2 * gChannels);
  float alpha = 1 - max(max(texColor.r, texColor.g), max(texColor.b, texColor.a));
  texColor = tex2D(TextureSampler0, tex0);
  
  return float4(texColor.rgb, texColor.a * alpha * gOpacity);
}

float4 DisplacementPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = tex2D(TextureSampler0, tex0 + (texColor2.rg - 0.5) * gChannels.rg);
  return float4(texColor.rgb, texColor.a * gOpacity);
}

float4 DisplacementMaskedPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = tex2D(TextureSampler0, tex0 + (texColor2.rg - 0.5) * gChannels.rg);
  return float4(texColor.rgb, texColor.a * (1 - gChannels.a + texColor2.a * gChannels.a) * gOpacity);
}

float4 DisplacementLightedPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = tex2D(TextureSampler0, tex0 + (texColor2.rg - 0.5) * gChannels.rg);
  float d = (0.5 - clamp(texColor2.b, 0, 0.5)) * 2;
  float l = (clamp(texColor2.b, 0.5, 1) - 0.5) * 2;
  return float4(texColor.rgb + (l - d) * gChannels.b, texColor.a * gOpacity);
}

float4 DisplacementLightedAndMaskedPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = tex2D(TextureSampler0, tex0 + (texColor2.rg - 0.5) * gChannels.rg);
  float d = (0.5 - clamp(texColor2.b, 0, 0.5)) * 2;
  float l = (clamp(texColor2.b, 0.5, 1) - 0.5) * 2;
  return float4(texColor.rgb + (l - d) * gChannels.b, texColor.a * (1 - gChannels.a + texColor2.a * gChannels.a) * gOpacity);
}

float4 AddPS(float2 tex0: TEXCOORD0, float4 color: COLOR0) : COLOR {
  float4 texColor2 = tex2D(TextureSampler1, tex0);
  float4 texColor = tex2D(TextureSampler0, tex0);
  
  return float4(texColor.rgb + texColor2.rgb, texColor.a + texColor2.a * gOpacity);
}


technique mask {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 MaskPS();
		// FillMode = wireframe;
		FillMode = solid;

		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique mask_inverted {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 MaskInvertedPS();
		// FillMode = wireframe;
		FillMode = solid;

		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique displacement {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 DisplacementPS();
		// FillMode = wireframe;
		FillMode = solid;

		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique displacement_masked {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 DisplacementMaskedPS();
		// FillMode = wireframe;
		FillMode = solid;
		
		// Alphablending
		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique displacement_lighted {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 DisplacementLightedPS();
		// FillMode = wireframe;
		FillMode = solid;
		
		// Alphablending
		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique displacement_lighted_and_masked {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 DisplacementLightedAndMaskedPS();
		// FillMode = wireframe;
		FillMode = solid;
		
		// Alphablending
		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}


technique add {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 AddPS();
		// FillMode = wireframe;
		FillMode = solid;
		
		// Alphablending
		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}